@@ -10,23 +10,28 @@ __metaclass__ = type
 
 
 import pytest
-
-from ansible_collections.community.internal_test_tools.tests.unit.compat.mock import MagicMock, patch
-
+from ansible_collections.community.internal_test_tools.tests.unit.compat.mock import (
+    MagicMock,
+    patch,
+)
 from ansible_collections.community.internal_test_tools.tests.unit.plugins.modules.utils import (
-    set_module_args,
-    ModuleTestCase,
     AnsibleExitJson,
     AnsibleFailJson,
+    ModuleTestCase,
+    set_module_args,
 )
 
 from ansible_collections.community.dns.plugins.modules import nameserver_record_info
 
 from ..module_utils.resolver_helper import (
-    mock_resolver,
-    mock_query_udp,
     create_mock_answer,
     create_mock_response,
+    mock_query_udp,
+    mock_resolver,
+)
+from ..module_utils.utils import (
+    patch_dict,
+    patch_dict_absent,
 )
 
 # We need dnspython
@@ -178,11 +183,11 @@ class TestNameserverRecordInfo(ModuleTestCase):
             with patch('dns.resolver.Resolver', resolver):
                 with patch('dns.query.udp', mock_query_udp(udp_sequence)):
                     with pytest.raises(AnsibleExitJson) as exc:
-                        set_module_args({
+                        with set_module_args({
                             'name': ['www.example.com'],
                             'type': 'TXT',
-                        })
-                        nameserver_record_info.main()
+                        }):
+                            nameserver_record_info.main()
 
         print(exc.value.args[0])
         assert exc.value.args[0]['changed'] is False
@@ -196,6 +201,12 @@ class TestNameserverRecordInfo(ModuleTestCase):
                 'value': 'asdf',
             }
         ]
+        assert exc.value.args[0]['results'][0]['result'][0]['entries'] == [
+            {
+                'strings': ['asdf'],
+                'value': 'asdf',
+            }
+        ]
         assert exc.value.args[0]['results'][0]['result'][1]['nameserver'] == 'ns.example.org'
         assert exc.value.args[0]['results'][0]['result'][1]['values'] == [
             {
@@ -207,6 +218,16 @@ class TestNameserverRecordInfo(ModuleTestCase):
                 'value': 'fdsa',
             },
         ]
+        assert exc.value.args[0]['results'][0]['result'][1]['entries'] == [
+            {
+                'strings': ['asdf'],
+                'value': 'asdf',
+            },
+            {
+                'strings': ['fdsa'],
+                'value': 'fdsa',
+            },
+        ]
 
     def test_timeout(self):
         fake_query = MagicMock()
@@ -319,13 +340,13 @@ class TestNameserverRecordInfo(ModuleTestCase):
             with patch('dns.resolver.Resolver', resolver):
                 with patch('dns.query.udp', mock_query_udp(udp_sequence)):
                     with pytest.raises(AnsibleFailJson) as exc:
-                        set_module_args({
+                        with set_module_args({
                             'name': ['www.example.com', 'mail.example.com'],
                             'type': 'TXT',
                             'query_timeout': 9,
                             'query_retry': 1,
-                        })
-                        nameserver_record_info.main()
+                        }):
+                            nameserver_record_info.main()
 
         print(exc.value.args[0])
         assert exc.value.args[0]['msg'] in (
@@ -343,13 +364,50 @@ class TestNameserverRecordInfo(ModuleTestCase):
                         'value': 'fdsaasdf',
                     }
                 ],
+                'entries': [
+                    {
+                        'strings': ['fdsa', 'asdf'],
+                        'value': 'fdsaasdf',
+                    }
+                ],
             }
         ]
         assert exc.value.args[0]['results'][1]['name'] == 'mail.example.com'
         assert exc.value.args[0]['results'][1]['result'] == []
 
     def test_nxdomain(self):
-        resolver = mock_resolver(['1.1.1.1'], {})
+        resolver = mock_resolver(['1.1.1.1'], {
+            ('1.1.1.1', ): [
+                {
+                    'target': 'ns.example.com',
+                    'rdtype': dns.rdatatype.A,
+                    'lifetime': 10,
+                    'result': create_mock_answer(dns.rrset.from_rdata(
+                        'ns.example.com',
+                        300,
+                        dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.A, '3.3.3.3'),
+                    )),
+                },
+                {
+                    'target': 'ns.example.com',
+                    'rdtype': dns.rdatatype.AAAA,
+                    'lifetime': 10,
+                    'result': create_mock_answer(dns.rrset.from_rdata(
+                        'ns.example.com',
+                        300,
+                        dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.AAAA, '1:2::3'),
+                    )),
+                },
+            ],
+            ('1:2::3', '3.3.3.3'): [
+                {
+                    'target': dns.name.from_unicode(u'www.example.com'),
+                    'rdtype': dns.rdatatype.TXT,
+                    'lifetime': 10,
+                    'result': create_mock_answer(rcode=dns.rcode.NXDOMAIN),
+                },
+            ],
+        })
         udp_sequence = [
             {
                 'query_target': dns.name.from_unicode(u'com'),
@@ -361,7 +419,20 @@ class TestNameserverRecordInfo(ModuleTestCase):
                 'result': create_mock_response(dns.rcode.NXDOMAIN),
             },
             {
-                'query_target': dns.name.from_unicode(u'com'),
+                'query_target': dns.name.from_unicode(u'example.com'),
+                'query_type': dns.rdatatype.NS,
+                'nameserver': '1.1.1.1',
+                'kwargs': {
+                    'timeout': 10,
+                },
+                'result': create_mock_response(dns.rcode.NOERROR, answer=[dns.rrset.from_rdata(
+                    'example.com',
+                    3600,
+                    dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.NS, 'ns.example.com'),
+                )]),
+            },
+            {
+                'query_target': dns.name.from_unicode(u'www.example.com'),
                 'query_type': dns.rdatatype.NS,
                 'nameserver': '1.1.1.1',
                 'kwargs': {
@@ -374,17 +445,107 @@ class TestNameserverRecordInfo(ModuleTestCase):
             with patch('dns.resolver.Resolver', resolver):
                 with patch('dns.query.udp', mock_query_udp(udp_sequence)):
                     with pytest.raises(AnsibleExitJson) as exc:
-                        set_module_args({
+                        with set_module_args({
                             'name': ['www.example.com'],
                             'type': 'TXT',
-                        })
-                        nameserver_record_info.main()
+                        }):
+                            nameserver_record_info.main()
 
         print(exc.value.args[0])
         assert exc.value.args[0]['changed'] is False
         assert len(exc.value.args[0]['results']) == 1
         assert exc.value.args[0]['results'][0]['name'] == 'www.example.com'
-        assert exc.value.args[0]['results'][0]['result'] == []
+        assert len(exc.value.args[0]['results'][0]['result']) == 1
+        assert exc.value.args[0]['results'][0]['result'][0]['nameserver'] == 'ns.example.com'
+        assert exc.value.args[0]['results'][0]['result'][0]['values'] == []
+        assert exc.value.args[0]['results'][0]['result'][0]['entries'] == []
+
+    def test_no_answer(self):
+        fake_query = MagicMock()
+        fake_query.question = 'Doctor Who?'
+        resolver = mock_resolver(['1.1.1.1'], {
+            ('1.1.1.1', ): [
+                {
+                    'target': 'ns.example.com',
+                    'rdtype': dns.rdatatype.A,
+                    'lifetime': 10,
+                    'result': create_mock_answer(dns.rrset.from_rdata(
+                        'ns.example.com',
+                        300,
+                        dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.A, '3.3.3.3'),
+                    )),
+                },
+                {
+                    'target': 'ns.example.com',
+                    'rdtype': dns.rdatatype.AAAA,
+                    'lifetime': 10,
+                    'result': create_mock_answer(dns.rrset.from_rdata(
+                        'ns.example.com',
+                        300,
+                        dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.AAAA, '1:2::3'),
+                    )),
+                },
+            ],
+            ('1:2::3', '3.3.3.3'): [
+                {
+                    'target': dns.name.from_unicode(u'www.example.com'),
+                    'rdtype': dns.rdatatype.TXT,
+                    'lifetime': 10,
+                    'raise': dns.resolver.NoAnswer(response=fake_query),
+                },
+            ],
+        })
+        udp_sequence = [
+            {
+                'query_target': dns.name.from_unicode(u'com'),
+                'query_type': dns.rdatatype.NS,
+                'nameserver': '1.1.1.1',
+                'kwargs': {
+                    'timeout': 10,
+                },
+                'result': create_mock_response(dns.rcode.NXDOMAIN),
+            },
+            {
+                'query_target': dns.name.from_unicode(u'example.com'),
+                'query_type': dns.rdatatype.NS,
+                'nameserver': '1.1.1.1',
+                'kwargs': {
+                    'timeout': 10,
+                },
+                'result': create_mock_response(dns.rcode.NOERROR, answer=[dns.rrset.from_rdata(
+                    'example.com',
+                    3600,
+                    dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.NS, 'ns.example.com'),
+                )]),
+            },
+            {
+                'query_target': dns.name.from_unicode(u'www.example.com'),
+                'query_type': dns.rdatatype.NS,
+                'nameserver': '1.1.1.1',
+                'kwargs': {
+                    'timeout': 10,
+                },
+                'result': create_mock_response(dns.rcode.NXDOMAIN),
+            },
+        ]
+        with patch('dns.resolver.get_default_resolver', resolver):
+            with patch('dns.resolver.Resolver', resolver):
+                with patch('dns.query.udp', mock_query_udp(udp_sequence)):
+                    with pytest.raises(AnsibleExitJson) as exc:
+                        with set_module_args({
+                            'name': ['www.example.com'],
+                            'type': 'TXT',
+                        }):
+                            nameserver_record_info.main()
+
+        print(exc.value.args[0])
+        assert exc.value.args[0]['changed'] is False
+        assert len(exc.value.args[0]['results']) == 1
+        assert exc.value.args[0]['results'][0]['name'] == 'www.example.com'
+        assert len(exc.value.args[0]['results'][0]['result']) == 1
+        assert exc.value.args[0]['results'][0]['result'][0]['nameserver'] == 'ns.example.com'
+        assert exc.value.args[0]['results'][0]['result'][0]['values'] == []
+        assert exc.value.args[0]['results'][0]['result'][0]['entries'] == []
 
     def test_servfail(self):
         resolver = mock_resolver(['1.1.1.1'], {})
@@ -403,11 +564,11 @@ class TestNameserverRecordInfo(ModuleTestCase):
             with patch('dns.resolver.Resolver', resolver):
                 with patch('dns.query.udp', mock_query_udp(udp_sequence)):
                     with pytest.raises(AnsibleFailJson) as exc:
-                        set_module_args({
+                        with set_module_args({
                             'name': ['www.example.com'],
                             'type': 'TXT',
-                        })
-                        nameserver_record_info.main()
+                        }):
+                            nameserver_record_info.main()
 
         print(exc.value.args[0])
         assert exc.value.args[0]['msg'] == 'Unexpected resolving error: Error SERVFAIL while querying 1.1.1.1 with query get NS for "com."'
@@ -533,14 +694,27 @@ class TestNameserverRecordInfo(ModuleTestCase):
             with patch('dns.resolver.Resolver', resolver):
                 with patch('dns.query.udp', mock_query_udp(udp_sequence)):
                     with pytest.raises(AnsibleFailJson) as exc:
-                        set_module_args({
+                        with set_module_args({
                             'name': ['www.example.com'],
                             'type': 'TXT',
-                        })
-                        nameserver_record_info.main()
+                        }):
+                            nameserver_record_info.main()
 
         print(exc.value.args[0])
         assert exc.value.args[0]['msg'] == 'Unexpected resolving error: Found CNAME loop starting at www.example.com'
         assert len(exc.value.args[0]['results']) == 1
         assert exc.value.args[0]['results'][0]['name'] == 'www.example.com'
         assert exc.value.args[0]['results'][0]['result'] == []
+
+    def test_unsupported_type(self):
+        with patch_dict(nameserver_record_info.NAME_TO_REQUIRED_VERSION, "A", "1.2.3"):
+            with patch_dict_absent(nameserver_record_info.NAME_TO_RDTYPE, "A"):
+                with pytest.raises(AnsibleFailJson) as exc:
+                    with set_module_args({
+                        'name': ['www.example.com'],
+                        'type': 'A',
+                    }):
+                        nameserver_record_info.main()
+
+        print(exc.value.args[0])
+        assert exc.value.args[0]['msg'] == 'Your dnspython version does not support A records. You need version 1.2.3 or newer.'
