@@ -4,8 +4,9 @@
 
 from __future__ import annotations
 
-import unittest
 from contextlib import contextmanager
+from io import StringIO
+from itertools import count
 from unittest.mock import patch
 
 from ansible_collections.community.internal_test_tools.tests.unit.plugins.modules.utils import (
@@ -16,9 +17,6 @@ from ansible_collections.community.internal_test_tools.tests.unit.plugins.module
 
 from ansible_collections.community.general.plugins.modules import keycloak_clientscope
 
-from io import StringIO
-from itertools import count
-
 
 @contextmanager
 def patch_keycloak_api(
@@ -650,7 +648,3 @@ class TestKeycloakAuthentication(ModuleTestCase):
 
         # Verify that the module's changed status matches what is expected
         self.assertIs(exec_info.exception.args[0]["changed"], changed)
-
-
-if __name__ == "__main__":
-    unittest.main()
