@@ -7,14 +7,13 @@ from __future__ import absolute_import, division, print_function
 __metaclass__ = type
 
 import unittest
-
-from unittest.mock import MagicMock, patch, call
+from unittest.mock import MagicMock, patch
 
 from ansible.module_utils import basic
 from ansible_collections.kubernetes.core.plugins.modules import helm
 from ansible_collections.kubernetes.core.tests.unit.utils.ansible_module_mock import (
-    AnsibleFailJson,
     AnsibleExitJson,
+    AnsibleFailJson,
     exit_json,
     fail_json,
     get_bin_path,
@@ -78,21 +77,26 @@ class TestDependencyUpdateWithoutChartRepoUrlOption(unittest.TestCase):
         helm.fetch_chart_info = MagicMock(return_value=self.chart_info_without_dep)
         helm.run_dep_update = MagicMock()
         with patch.object(basic.AnsibleModule, "run_command") as mock_run_command:
-            mock_run_command.return_value = (
-                0,
-                "configuration updated",
-                "",
-            )  # successful execution
+            # Mock responses: first call is helm version, second is the actual command
+            mock_run_command.side_effect = [
+                (
+                    0,
+                    'version.BuildInfo{Version:"v3.10.0", GitCommit:"", GoVersion:"go1.18"}',
+                    "",
+                ),
+                (0, "configuration updated", ""),
+            ]
             with self.assertRaises(AnsibleExitJson) as result:
                 helm.main()
         helm.run_dep_update.assert_not_called()
-        mock_run_command.assert_called_once_with(
-            "/usr/bin/helm upgrade -i --reset-values test /tmp/path",
-            environ_update={"HELM_NAMESPACE": "test"},
+        # Check the last call (actual helm command, after version check)
+        assert (
+            mock_run_command.call_args_list[-1][0][0]
+            == "/usr/bin/helm upgrade -i --reset-values test '/tmp/path'"
         )
         assert (
             result.exception.args[0]["command"]
-            == "/usr/bin/helm upgrade -i --reset-values test /tmp/path"
+            == "/usr/bin/helm upgrade -i --reset-values test '/tmp/path'"
         )
 
     def test_dependency_update_option_false(self):
@@ -108,21 +112,26 @@ class TestDependencyUpdateWithoutChartRepoUrlOption(unittest.TestCase):
         helm.fetch_chart_info = MagicMock(return_value=self.chart_info_without_dep)
         helm.run_dep_update = MagicMock()
         with patch.object(basic.AnsibleModule, "run_command") as mock_run_command:
-            mock_run_command.return_value = (
-                0,
-                "configuration updated",
-                "",
-            )  # successful execution
+            # Mock responses: first call is helm version, second is the actual command
+            mock_run_command.side_effect = [
+                (
+                    0,
+                    'version.BuildInfo{Version:"v3.10.0", GitCommit:"", GoVersion:"go1.18"}',
+                    "",
+                ),
+                (0, "configuration updated", ""),
+            ]
             with self.assertRaises(AnsibleExitJson) as result:
                 helm.main()
         helm.run_dep_update.assert_not_called()
-        mock_run_command.assert_called_once_with(
-            "/usr/bin/helm upgrade -i --reset-values test /tmp/path",
-            environ_update={"HELM_NAMESPACE": "test"},
+        # Check the last call (actual helm command, after version check)
+        assert (
+            mock_run_command.call_args_list[-1][0][0]
+            == "/usr/bin/helm upgrade -i --reset-values test '/tmp/path'"
         )
         assert (
             result.exception.args[0]["command"]
-            == "/usr/bin/helm upgrade -i --reset-values test /tmp/path"
+            == "/usr/bin/helm upgrade -i --reset-values test '/tmp/path'"
         )
 
     def test_dependency_update_option_true(self):
@@ -138,22 +147,27 @@ class TestDependencyUpdateWithoutChartRepoUrlOption(unittest.TestCase):
         helm.fetch_chart_info = MagicMock(return_value=self.chart_info_with_dep)
 
         with patch.object(basic.AnsibleModule, "run_command") as mock_run_command:
-            mock_run_command.return_value = 0, "configuration updated", ""
+            # Mock responses: first call is helm version, second is the actual command
+            mock_run_command.side_effect = [
+                (
+                    0,
+                    'version.BuildInfo{Version:"v3.10.0", GitCommit:"", GoVersion:"go1.18"}',
+                    "",
+                ),
+                (0, "configuration updated", ""),
+            ]
             with patch.object(basic.AnsibleModule, "warn") as mock_warn:
                 with self.assertRaises(AnsibleExitJson) as result:
                     helm.main()
         mock_warn.assert_not_called()
-        mock_run_command.assert_has_calls(
-            [
-                call(
-                    "/usr/bin/helm upgrade -i --reset-values test /tmp/path",
-                    environ_update={"HELM_NAMESPACE": "test"},
-                )
-            ]
+        # Check calls include the actual helm command (after version check)
+        assert any(
+            "/usr/bin/helm upgrade -i --reset-values test '/tmp/path'" in str(call)
+            for call in mock_run_command.call_args_list
         )
         assert (
             result.exception.args[0]["command"]
-            == "/usr/bin/helm upgrade -i --reset-values test /tmp/path"
+            == "/usr/bin/helm upgrade -i --reset-values test '/tmp/path'"
         )
 
     def test_dependency_update_option_true_without_dependencies_block(self):
@@ -168,26 +182,27 @@ class TestDependencyUpdateWithoutChartRepoUrlOption(unittest.TestCase):
         helm.get_release_status = MagicMock(return_value=None)
         helm.fetch_chart_info = MagicMock(return_value=self.chart_info_without_dep)
         with patch.object(basic.AnsibleModule, "run_command") as mock_run_command:
-            mock_run_command.return_value = (
-                0,
-                "configuration updated",
-                "",
-            )  # successful execution
+            # Mock responses: first call is helm version, second is the actual command
+            mock_run_command.side_effect = [
+                (
+                    0,
+                    'version.BuildInfo{Version:"v3.10.0", GitCommit:"", GoVersion:"go1.18"}',
+                    "",
+                ),
+                (0, "configuration updated", ""),
+            ]
             with patch.object(basic.AnsibleModule, "warn") as mock_warn:
                 with self.assertRaises(AnsibleExitJson) as result:
                     helm.main()
         mock_warn.assert_called_once()
-        mock_run_command.assert_has_calls(
-            [
-                call(
-                    "/usr/bin/helm upgrade -i --reset-values test /tmp/path",
-                    environ_update={"HELM_NAMESPACE": "test"},
-                )
-            ]
+        # Check calls include the actual helm command (after version check)
+        assert any(
+            "/usr/bin/helm upgrade -i --reset-values test '/tmp/path'" in str(call)
+            for call in mock_run_command.call_args_list
         )
         assert (
             result.exception.args[0]["command"]
-            == "/usr/bin/helm upgrade -i --reset-values test /tmp/path"
+            == "/usr/bin/helm upgrade -i --reset-values test '/tmp/path'"
         )
 
 
@@ -242,20 +257,25 @@ class TestDependencyUpdateWithChartRepoUrlOption(unittest.TestCase):
         helm.get_release_status = MagicMock(return_value=None)
         helm.fetch_chart_info = MagicMock(return_value=self.chart_info_without_dep)
         with patch.object(basic.AnsibleModule, "run_command") as mock_run_command:
-            mock_run_command.return_value = (
-                0,
-                "configuration updated",
-                "",
-            )  # successful execution
+            # Mock responses: first call is helm version, second is the actual command
+            mock_run_command.side_effect = [
+                (
+                    0,
+                    'version.BuildInfo{Version:"v3.10.0", GitCommit:"", GoVersion:"go1.18"}',
+                    "",
+                ),
+                (0, "configuration updated", ""),
+            ]
             with self.assertRaises(AnsibleExitJson) as result:
                 helm.main()
-        mock_run_command.assert_called_once_with(
-            "/usr/bin/helm --repo=http://repo.example/charts upgrade -i --reset-values test chart1",
-            environ_update={"HELM_NAMESPACE": "test"},
+        # Check the last call (actual helm command, after version check)
+        assert (
+            mock_run_command.call_args_list[-1][0][0]
+            == "/usr/bin/helm --repo=http://repo.example/charts upgrade -i --reset-values test 'chart1'"
         )
         assert (
             result.exception.args[0]["command"]
-            == "/usr/bin/helm --repo=http://repo.example/charts upgrade -i --reset-values test chart1"
+            == "/usr/bin/helm --repo=http://repo.example/charts upgrade -i --reset-values test 'chart1'"
         )
 
     def test_dependency_update_option_False(self):
@@ -271,20 +291,25 @@ class TestDependencyUpdateWithChartRepoUrlOption(unittest.TestCase):
         helm.get_release_status = MagicMock(return_value=None)
         helm.fetch_chart_info = MagicMock(return_value=self.chart_info_without_dep)
         with patch.object(basic.AnsibleModule, "run_command") as mock_run_command:
-            mock_run_command.return_value = (
-                0,
-                "configuration updated",
-                "",
-            )  # successful execution
+            # Mock responses: first call is helm version, second is the actual command
+            mock_run_command.side_effect = [
+                (
+                    0,
+                    'version.BuildInfo{Version:"v3.10.0", GitCommit:"", GoVersion:"go1.18"}',
+                    "",
+                ),
+                (0, "configuration updated", ""),
+            ]
             with self.assertRaises(AnsibleExitJson) as result:
                 helm.main()
-        mock_run_command.assert_called_once_with(
-            "/usr/bin/helm --repo=http://repo.example/charts upgrade -i --reset-values test chart1",
-            environ_update={"HELM_NAMESPACE": "test"},
+        # Check the last call (actual helm command, after version check)
+        assert (
+            mock_run_command.call_args_list[-1][0][0]
+            == "/usr/bin/helm --repo=http://repo.example/charts upgrade -i --reset-values test 'chart1'"
         )
         assert (
             result.exception.args[0]["command"]
-            == "/usr/bin/helm --repo=http://repo.example/charts upgrade -i --reset-values test chart1"
+            == "/usr/bin/helm --repo=http://repo.example/charts upgrade -i --reset-values test 'chart1'"
         )
 
     def test_dependency_update_option_True_and_replace_option_disabled(self):
@@ -300,11 +325,15 @@ class TestDependencyUpdateWithChartRepoUrlOption(unittest.TestCase):
         helm.get_release_status = MagicMock(return_value=None)
         helm.fetch_chart_info = MagicMock(return_value=self.chart_info_with_dep)
         with patch.object(basic.AnsibleModule, "run_command") as mock_run_command:
-            mock_run_command.return_value = (
-                0,
-                "configuration updated",
-                "",
-            )  # successful execution
+            # Mock responses: first call is helm version, second is the actual command
+            mock_run_command.side_effect = [
+                (
+                    0,
+                    'version.BuildInfo{Version:"v3.10.0", GitCommit:"", GoVersion:"go1.18"}',
+                    "",
+                ),
+                (0, "configuration updated", ""),
+            ]
             with self.assertRaises(AnsibleFailJson) as result:
                 helm.main()
         # mock_run_command.assert_called_once_with('/usr/bin/helm --repo=http://repo.example/charts upgrade -i --reset-values test chart1',
@@ -329,20 +358,25 @@ class TestDependencyUpdateWithChartRepoUrlOption(unittest.TestCase):
         helm.get_release_status = MagicMock(return_value=None)
         helm.fetch_chart_info = MagicMock(return_value=self.chart_info_without_dep)
         with patch.object(basic.AnsibleModule, "run_command") as mock_run_command:
-            mock_run_command.return_value = (
-                0,
-                "configuration updated",
-                "",
-            )  # successful execution
+            # Mock responses: first call is helm version, second is the actual command
+            mock_run_command.side_effect = [
+                (
+                    0,
+                    'version.BuildInfo{Version:"v3.10.0", GitCommit:"", GoVersion:"go1.18"}',
+                    "",
+                ),
+                (0, "configuration updated", ""),
+            ]
             with self.assertRaises(AnsibleExitJson) as result:
                 helm.main()
-        mock_run_command.assert_called_once_with(
-            "/usr/bin/helm --repo=http://repo.example/charts install --dependency-update --replace test chart1",
-            environ_update={"HELM_NAMESPACE": "test"},
+        # Check the last call (actual helm command, after version check)
+        assert (
+            mock_run_command.call_args_list[-1][0][0]
+            == "/usr/bin/helm --repo=http://repo.example/charts install --dependency-update --replace test 'chart1'"
         )
         assert (
             result.exception.args[0]["command"]
-            == "/usr/bin/helm --repo=http://repo.example/charts install --dependency-update --replace test chart1"
+            == "/usr/bin/helm --repo=http://repo.example/charts install --dependency-update --replace test 'chart1'"
         )
 
 
@@ -396,20 +430,25 @@ class TestDependencyUpdateWithChartRefIsUrl(unittest.TestCase):
         helm.get_release_status = MagicMock(return_value=None)
         helm.fetch_chart_info = MagicMock(return_value=self.chart_info_without_dep)
         with patch.object(basic.AnsibleModule, "run_command") as mock_run_command:
-            mock_run_command.return_value = (
-                0,
-                "configuration updated",
-                "",
-            )  # successful execution
+            # Mock responses: first call is helm version, second is the actual command
+            mock_run_command.side_effect = [
+                (
+                    0,
+                    'version.BuildInfo{Version:"v3.10.0", GitCommit:"", GoVersion:"go1.18"}',
+                    "",
+                ),
+                (0, "configuration updated", ""),
+            ]
             with self.assertRaises(AnsibleExitJson) as result:
                 helm.main()
-        mock_run_command.assert_called_once_with(
-            "/usr/bin/helm upgrade -i --reset-values test http://repo.example/charts/application.tgz",
-            environ_update={"HELM_NAMESPACE": "test"},
+        # Check the last call (actual helm command, after version check)
+        assert (
+            mock_run_command.call_args_list[-1][0][0]
+            == "/usr/bin/helm upgrade -i --reset-values test 'http://repo.example/charts/application.tgz'"
         )
         assert (
             result.exception.args[0]["command"]
-            == "/usr/bin/helm upgrade -i --reset-values test http://repo.example/charts/application.tgz"
+            == "/usr/bin/helm upgrade -i --reset-values test 'http://repo.example/charts/application.tgz'"
         )
 
     def test_dependency_update_option_False(self):
@@ -424,20 +463,25 @@ class TestDependencyUpdateWithChartRefIsUrl(unittest.TestCase):
         helm.get_release_status = MagicMock(return_value=None)
         helm.fetch_chart_info = MagicMock(return_value=self.chart_info_without_dep)
         with patch.object(basic.AnsibleModule, "run_command") as mock_run_command:
-            mock_run_command.return_value = (
-                0,
-                "configuration updated",
-                "",
-            )  # successful execution
+            # Mock responses: first call is helm version, second is the actual command
+            mock_run_command.side_effect = [
+                (
+                    0,
+                    'version.BuildInfo{Version:"v3.10.0", GitCommit:"", GoVersion:"go1.18"}',
+                    "",
+                ),
+                (0, "configuration updated", ""),
+            ]
             with self.assertRaises(AnsibleExitJson) as result:
                 helm.main()
-        mock_run_command.assert_called_once_with(
-            "/usr/bin/helm upgrade -i --reset-values test http://repo.example/charts/application.tgz",
-            environ_update={"HELM_NAMESPACE": "test"},
+        # Check the last call (actual helm command, after version check)
+        assert (
+            mock_run_command.call_args_list[-1][0][0]
+            == "/usr/bin/helm upgrade -i --reset-values test 'http://repo.example/charts/application.tgz'"
         )
         assert (
             result.exception.args[0]["command"]
-            == "/usr/bin/helm upgrade -i --reset-values test http://repo.example/charts/application.tgz"
+            == "/usr/bin/helm upgrade -i --reset-values test 'http://repo.example/charts/application.tgz'"
         )
 
     def test_dependency_update_option_True_and_replace_option_disabled(self):
@@ -452,11 +496,15 @@ class TestDependencyUpdateWithChartRefIsUrl(unittest.TestCase):
         helm.get_release_status = MagicMock(return_value=None)
         helm.fetch_chart_info = MagicMock(return_value=self.chart_info_with_dep)
         with patch.object(basic.AnsibleModule, "run_command") as mock_run_command:
-            mock_run_command.return_value = (
-                0,
-                "configuration updated",
-                "",
-            )  # successful execution
+            # Mock responses: first call is helm version, second is the actual command
+            mock_run_command.side_effect = [
+                (
+                    0,
+                    'version.BuildInfo{Version:"v3.10.0", GitCommit:"", GoVersion:"go1.18"}',
+                    "",
+                ),
+                (0, "configuration updated", ""),
+            ]
             with self.assertRaises(AnsibleFailJson) as result:
                 helm.main()
         # mock_run_command.assert_called_once_with('/usr/bin/helm --repo=http://repo.example/charts upgrade -i --reset-values test chart1',
@@ -480,18 +528,23 @@ class TestDependencyUpdateWithChartRefIsUrl(unittest.TestCase):
         helm.get_release_status = MagicMock(return_value=None)
         helm.fetch_chart_info = MagicMock(return_value=self.chart_info_without_dep)
         with patch.object(basic.AnsibleModule, "run_command") as mock_run_command:
-            mock_run_command.return_value = (
-                0,
-                "configuration updated",
-                "",
-            )  # successful execution
+            # Mock responses: first call is helm version, second is the actual command
+            mock_run_command.side_effect = [
+                (
+                    0,
+                    'version.BuildInfo{Version:"v3.10.0", GitCommit:"", GoVersion:"go1.18"}',
+                    "",
+                ),
+                (0, "configuration updated", ""),
+            ]
             with self.assertRaises(AnsibleExitJson) as result:
                 helm.main()
-        mock_run_command.assert_called_once_with(
-            "/usr/bin/helm install --dependency-update --replace test http://repo.example/charts/application.tgz",
-            environ_update={"HELM_NAMESPACE": "test"},
+        # Check the last call (actual helm command, after version check)
+        assert (
+            mock_run_command.call_args_list[-1][0][0]
+            == "/usr/bin/helm install --dependency-update --replace test 'http://repo.example/charts/application.tgz'"
         )
         assert (
             result.exception.args[0]["command"]
-            == "/usr/bin/helm install --dependency-update --replace test http://repo.example/charts/application.tgz"
+            == "/usr/bin/helm install --dependency-update --replace test 'http://repo.example/charts/application.tgz'"
         )
